from pokie_mail.constants import STATUS_DRAFT
from pokie_mail.helpers import MessageBuilder
from pokie_mail.service import MessageTemplateService
from tests.unit.constants import TPL_LOCALE, TPL_2_NAME


class TestBuilder:
    def test_builder(
        self, template_1, template_2, svc_template: MessageTemplateService
    ):
        # add template
        svc_template.insert(template_1)
        svc_template.insert(template_2)

        # list templates, should have 2
        rows, items = svc_template.list(None)
        assert rows == 2

        builder = svc_template.get_builder(TPL_2_NAME, TPL_LOCALE, 0)
        assert isinstance(builder, MessageBuilder)

        # assemble message from template, with placeholders
        record = builder.assemble(
            msg_from="origin",
            msg_to="destination",
            data={"{{color}}": "brown", "{{animal}}": "fox", "{{what}}": "jumps"},
        )
        assert record.sent is None
        assert record.status == STATUS_DRAFT
        assert record.msg_to == "destination"
        assert record.msg_from == "origin"
        assert record.html == "the quick brown fox jumps over the lazy dog"
        assert record.content == record.html
        assert record.channel == 0

        # assemble message with default placeholders
        record = builder.assemble(msg_from="origin", msg_to="destination", data={})
        assert record.sent is None
        assert record.status == STATUS_DRAFT
        assert record.msg_to == "destination"
        assert record.msg_from == "origin"
        assert record.html == "the quick blue duck flies over the lazy dog"
        assert record.content == record.html



import pytest

from pokie_mail.constants import SVC_MESSAGE_TEMPLATE, SVC_MESSAGE_QUEUE
from pokie_mail.dto import MessageTemplateRecord
from pokie_mail.service import MessageTemplateService, MessageQueueService
from tests.unit.constants import TPL_1_NAME, TPL_LOCALE, TPL_2_NAME


@pytest.fixture
def svc_template(pokie_service_manager) -> MessageTemplateService:
    return pokie_service_manager.get(SVC_MESSAGE_TEMPLATE)


@pytest.fixture
def svc_queue(pokie_service_manager) -> MessageQueueService:
    return pokie_service_manager.get(SVC_MESSAGE_QUEUE)


@pytest.fixture
def template_1() -> MessageTemplateRecord:
    return MessageTemplateRecord(
        template=TPL_1_NAME,
        language=TPL_LOCALE,
        channel=0,
        label="label for template 1",
        subject="template subject",
        body="",
        html="",
        placeholders="",
    )


@pytest.fixture
def template_2() -> MessageTemplateRecord:
    return MessageTemplateRecord(
        template=TPL_2_NAME,
        language=TPL_LOCALE,
        channel=0,
        label="label for base_system_template",
        subject="template subject",
        body="the quick {{color}} {{animal}} {{what}} over the lazy dog",
        html="the quick {{color}} {{animal}} {{what}} over the lazy dog",
        placeholders={"{{color}}": "blue", "{{animal}}": "duck", "{{what}}": "flies"},
    )



from pokie_mail.constants import STATUS_DRAFT, STATUS_SENT, STATUS_QUEUED
from pokie_mail.helpers import MessageBuilder
from pokie_mail.service import MessageTemplateService, MessageQueueService
from tests.unit.constants import TPL_LOCALE, TPL_2_NAME


class TestQueue:
    def test_queue(
        self,
        template_2,
        svc_queue: MessageQueueService,
        svc_template: MessageTemplateService,
    ):
        # add template
        svc_template.insert(template_2)

        builder = svc_template.get_builder(TPL_2_NAME, TPL_LOCALE, 0)
        assert isinstance(builder, MessageBuilder)

        # assemble message from template, with placeholders
        record = builder.assemble(
            msg_from="origin",
            msg_to="destination",
            data={"{{color}}": "brown", "{{animal}}": "fox", "{{what}}": "jumps"},
        )
        assert record.sent is None
        assert record.status == STATUS_DRAFT
        assert record.msg_to == "destination"
        assert record.msg_from == "origin"
        assert record.html == "the quick brown fox jumps over the lazy dog"
        assert record.content == record.html
        assert record.channel == 0

        # queue message
        msg_id = svc_queue.queue(record)

        # fetch for processing
        msg_to_send = svc_queue.fetch_for_processing(0)
        assert msg_to_send is not None
        assert msg_id == msg_to_send.id

        # simulate the sending of message
        svc_queue.update_status(msg_id, STATUS_SENT)
        msg_to_send = svc_queue.fetch_for_processing(0)
        assert msg_to_send is None

        results = svc_queue.list_by_status(0, STATUS_SENT)
        assert len(results) == 1

        results = svc_queue.list_by_status(0, STATUS_QUEUED)
        assert len(results) == 0



# Version
POKIE_MAIL_VERSION = ["1", "0", "1"]


def get_version():
    return ".".join(POKIE_MAIL_VERSION)


# Service names

SVC_MESSAGE_QUEUE = "sv_pokie_mail_msg_queue"
SVC_MESSAGE_TEMPLATE = "sv_pokie_mail_msg_template"

# MessageQueueRecord status
STATUS_DRAFT = "D"  # not committed to database yet
STATUS_QUEUED = "Q"
STATUS_LOCKED = "L"
STATUS_FAILED = "F"
STATUS_SENT = "S"

VALID_STATUS = [
    STATUS_QUEUED,
    STATUS_LOCKED,
    STATUS_FAILED,
    STATUS_SENT,
]

# communication channels

CHANNEL_SMTP = 0

# SMTP Configuration variables
CFG_SMTP_HOST = "smtp_host"
CFG_SMTP_PORT = "smtp_port"
CFG_SMTP_USE_TLS = "smtp_use_tls"
CFG_SMTP_USE_SSL = "smtp_use_ssl"
CFG_SMTP_DEBUG = "smtp_debug"
CFG_SMTP_USERNAME = "smtp_username"
CFG_SMTP_PASSWORD = "smtp_password"
CFG_SMTP_DEFAULT_SENDER = "smtp_default_sender"
CFG_SMTP_TIMEOUT = "smtp_timeout"
CFG_SMTP_SSL_KEYFILE = "smtp_ssl_keyfile"
CFG_SMTP_SSL_CERTFILE = "smtp_ssl_certfile"



import logging

from rick.base import Di
from rick.mixin import Injectable, Runnable
from rick.resource.console import ConsoleWriter
from rick_mailer import Mailer, SMTPFactory

from pokie.constants import DI_TTY, DI_SERVICES, DI_CONFIG
from pokie_mail.constants import (
    SVC_MESSAGE_QUEUE,
    CHANNEL_SMTP,
    STATUS_SENT,
    STATUS_FAILED,
)
from pokie_mail.service import MessageQueueService


class MailJob(Injectable, Runnable):
    DEFAULT_MAILS_PER_RUN = 10000

    def __init__(self, di: Di):
        super().__init__(di)
        self.tty = di.get(DI_TTY) if di.has(DI_TTY) else ConsoleWriter()
        self.mails_per_run = di.get(DI_CONFIG).get(
            "mails_per_run", self.DEFAULT_MAILS_PER_RUN
        )

    @property
    def get_smtp_mailer(self) -> Mailer:
        config = self.get_di().get(DI_CONFIG).asdict()
        return Mailer(SMTPFactory(config))

    @property
    def svc_queue(self) -> MessageQueueService:
        return self.get_di().get(DI_SERVICES).get(SVC_MESSAGE_QUEUE)


class MailQueueJob(MailJob):
    def run_smtp(self):
        total = 0
        while total < self.mails_per_run:
            record = self.svc_queue.fetch_for_processing(CHANNEL_SMTP)
            if record is None:
                # no more messages, exit
                return True

            try:
                if (
                    self.get_smtp_mailer.send_mail(
                        record.title,
                        record.content,
                        record.msg_from,
                        record.msg_to.split(","),
                        html_message=record.html,
                    )
                    > 0
                ):
                    self.svc_queue.update_status(record.id, STATUS_SENT)
            except Exception as e:
                self.tty.error(str(e))
                self.svc_queue.update_status(record.id, STATUS_FAILED)

        return True

    def run(self, di: Di):
        # run smtp - channel 0
        return self.run_smtp()



from pokie_mail.dto import MessageTemplateRecord
from pokie_mail.helpers import MessageBuilder
from pokie_mail.service import MessageTemplateService
from tests.unit.constants import TPL_LOCALE, TPL_1_NAME


class TestTemplate:
    def test_template_add(self, template_1, svc_template: MessageTemplateService):
        # list templates, should be empty
        rows, items = svc_template.list(None)
        assert rows == 0

        # add template
        svc_template.insert(template_1)

        # list templates, should have 1
        rows, items = svc_template.list(None)
        assert rows == 1

        # read a valid template
        tpl = svc_template.get_template(TPL_1_NAME, TPL_LOCALE, 0)
        assert isinstance(tpl, MessageTemplateRecord)

        # read an invalid template - invalid channel
        tpl = svc_template.get_template(TPL_1_NAME, TPL_LOCALE, 1)
        assert tpl is None

    def test_template_get_builder(
        self, template_1, template_2, svc_template: MessageTemplateService
    ):
        # add template
        svc_template.insert(template_1)
        svc_template.insert(template_2)

        # list templates, should have 2
        rows, items = svc_template.list(None)
        assert rows == 2

        builder = svc_template.get_builder(TPL_1_NAME, TPL_LOCALE, 0)
        assert isinstance(builder, MessageBuilder)



from typing import Optional
from pokie.rest import RestService

from pokie_mail.constants import CHANNEL_SMTP
from pokie_mail.dto import MessageTemplateRecord
from pokie_mail.helpers import MessageBuilder
from pokie_mail.repository import MessageTemplateRepository


class MessageTemplateService(RestService):
    record_class = MessageTemplateRecord
    repository_class = MessageTemplateRepository

    def get_template(
        self, template: str, locale: str, channel: int = CHANNEL_SMTP
    ) -> Optional[MessageTemplateRecord]:
        return self.repository.find_template(template, locale, channel)

    def get_builder(
        self, template: str, locale: str, channel: int = CHANNEL_SMTP
    ) -> Optional[MessageBuilder]:
        template = self.repository.find_template(template, locale, channel)
        if template is None:
            return None
        return MessageBuilder(template)



from typing import List, Optional

from pokie.constants import DI_DB

from pokie_mail.constants import STATUS_QUEUED, VALID_STATUS
from pokie_mail.dto import MessageQueueRecord
from pokie_mail.repository import MessageQueueRepository
from rick.mixin import Injectable


class MessageQueueService(Injectable):
    def queue(self, record: MessageQueueRecord) -> str:
        record.status = STATUS_QUEUED
        return self.repo_queue.insert_pk(record)

    def list_by_status(
        self, channel: int, status: str, limit: int = 1000
    ) -> List[MessageQueueRecord]:
        return self.repo_queue.find_by_status(channel, status, limit)

    def valid_status(self, status: str) -> bool:
        return status in VALID_STATUS

    def update_status(self, id_record: str, status: str, raise_on_error=True):
        if not self.valid_status(status):
            if raise_on_error:
                raise RuntimeError(
                    "MessageQueueService.update_status(): invalid status '{}'".format(
                        status
                    )
                )
            return
        self.repo_queue.update_status(id_record, status)

    def fetch_for_processing(self, channel: int) -> Optional[MessageQueueRecord]:
        """
        Pick & lock a message for processing
        :return: MessageQueueRecord or None
        """
        return self.repo_queue.find_first_and_lock(channel, STATUS_QUEUED)

    def purge(self):
        """
        Removes all queued records
        :return:
        """
        return self.repo_queue.truncate()

    @property
    def repo_queue(self) -> MessageQueueRepository:
        return MessageQueueRepository(self.get_di().get(DI_DB))



from rick.resource.config import StrOrFile


# pokie_mail SMTP configuration
class MailConfigTemplate:
    # number of emails to send at once
    MAILS_PER_RUN = 10000

    # Message channel configuration
    channels = {"0": "SMTP"}
    SMTP_HOST = "localhost"
    SMTP_PORT = 25
    SMTP_USE_TLS = False
    SMTP_USE_SSL = False
    SMTP_DEBUG = False
    SMTP_USERNAME = StrOrFile("username")
    SMTP_PASSWORD = StrOrFile("password")
    SMTP_DEFAULT_SENDER = None
    SMTP_TIMEOUT = None
    SMTP_SSL_KEYFILE = None
    SMTP_SSL_CERTFILE = None
